"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFunctionParameterNames = exports.getRandomString = exports.base64Decode = exports.base64Encode = void 0;
var crypto_1 = require("crypto");
/**
 * convert a string to a utf-8 encoded base64 string
 *
 * @param source the source string to encode
 * @returns the base64 encoded string
 */
function base64Encode(source) {
    return Buffer.from(new TextEncoder().encode(source)).toString('base64');
}
exports.base64Encode = base64Encode;
/**
 * convert a utf-8 encoded base64 string to a string
 *
 * @param source the source base64 string to decode
 * @returns the decoded string
 */
function base64Decode(source) {
    return new TextDecoder('utf8').decode(Buffer.from(source, 'base64'));
}
exports.base64Decode = base64Decode;
/**
 * create a new random string.
 * the string will only contain alphanumerical characters ([A-Za-z0-9])
 *
 * @param length the length of the string.
 *               if not even, the length is shortened by one character
 * @returns a random string
 */
function getRandomString(length) {
    return (0, crypto_1.randomBytes)(length / 2).toString('hex');
}
exports.getRandomString = getRandomString;
/**
 * get the parameter names of a function or method
 *
 * @param fn the function to get the parameter names of
 * @returns a array containing the parameter names of the function, in order
 */
// eslint-disable-next-line @typescript-eslint/ban-types
function getFunctionParameterNames(fn) {
    // convert method body to a string, reduce to a single line
    var methodDefinition = fn.toString().replace(/\r?\n/g, '');
    // prepare regex with method name embedded
    var regex = new RegExp("^(?:async\\s)?(?:function\\s)?(?:".concat(fn.name, ")?\\s*\\(([\\w\\s,_]*)\\)\\s*{"));
    // get the parameter list of the method
    var parameterListMatch = methodDefinition.match(regex);
    if (!parameterListMatch || parameterListMatch.length != 2) {
        throw new Error('failed to parse method parameter list');
    }
    // split parameter list on commas and trim the resulting parameter names
    return parameterListMatch[1]
        .split(',')
        .map(function (p) { return p.trim(); })
        .filter(function (p) { return p; });
}
exports.getFunctionParameterNames = getFunctionParameterNames;
//# sourceMappingURL=util.js.map